#include <ShiftedLCD.h>
#include <SPI.h>
#include <IRremote.h>
#include "EEPROMAnything.h"
#include "DG409.h"
#include "PGA2310.h"

#define MAX_GAIN 192
#define MIN_GAIN 0

#define POWER_CODE 0x7715D633

#define VOLUME_UP_CODE 0xc1a2533e
#define VOLUME_DOWN_CODE 0xb92a5988

#define INPUT_1 0x150d53e6
#define INPUT_2 0x94b01292
#define INPUT_3 0xdd200394
#define INPUT_4 0x988b49d8
#define INPUT_5 0x1bf06be

typedef enum {
  GainUp = 1,
  GainDown = 2
} GainChange;

typedef enum {
  InputAnalog,
  InputOptical,
  InputCoaxial,
  InputAirplay,
  InputBluetooth
} Input;

const char* AnalogInputTitle = "       Analog   ";
const char* OpticalInputTitle = "       Optical  ";
const char* CoaxialInputTitle = "       Coaxial  ";
const char* AirplayInputTitle = "       Airplay  ";
const char* BluetoothInputTitle = "       Bluetooth";

const int SelectorPin0 = 8;
const int SelectorPin1 = 9;

const int IRReceiverPin = 14; 
const int LCDPin = 7;
const int VolumeControlPin = 10;

const int PowerPin = 3;

const int MutePin = 2;

struct config_t {
  int gain = 32;
  int activeInput = 0;
} configuration;

LiquidCrystal lcd(LCDPin);

IRrecv irReceiver(IRReceiverPin);
decode_results results;

DG409 inputSelector(SelectorPin0, SelectorPin1);
PGA2310 volumeControl(VolumeControlPin);

void printEmptyLine() {
  lcd.print("                ");
}

void printGain(int gain) {
  lcd.home();
  printEmptyLine();
  lcd.home();
  lcd.print("Volume: ");
  lcd.print(gain);
}

void printInput(Input input) {
  lcd.setCursor(0, 1);
  printEmptyLine();
  lcd.setCursor(0, 1);

  switch(input) {
     case InputAnalog:
     lcd.print(AnalogInputTitle);
     break;
    case InputOptical:
     lcd.print(OpticalInputTitle);
     break;
    case InputCoaxial:
     lcd.print(CoaxialInputTitle);
     break;
    case InputAirplay:
     lcd.print(AirplayInputTitle);
     break;
    case InputBluetooth:
     lcd.print(BluetoothInputTitle);
     break;
  }
}

void changeGain(GainChange change) {
  switch(change) {
    case GainUp:
      configuration.gain = constrain(configuration.gain + 1, MIN_GAIN, MAX_GAIN);
      break;
    case GainDown:
      configuration.gain = constrain(configuration.gain - 1, MIN_GAIN, MAX_GAIN);
      break;
  }
  volumeControl.setGain(configuration.gain);
  printGain(configuration.gain);        
  EEPROM_writeAnything(0, configuration);
}

void changeInput(Input input) {

  if (configuration.activeInput == input) {
    return;
  }

  configuration.activeInput = input;
  
  switch(input) {
    case InputAnalog:
     inputSelector.selectInput(0);
     break;
    case InputOptical:
     inputSelector.selectInput(1);
     break;
    case InputCoaxial:
     inputSelector.selectInput(1);
     break;
    case InputAirplay:
     inputSelector.selectInput(2);
     break;
    case InputBluetooth:
     inputSelector.selectInput(3);
     break;
  }
  printInput(input);

  EEPROM_writeAnything(0, configuration);
}

int powerState = 0;
unsigned long powerFlipTime = 0;
unsigned long powerFlipRequestTime = 0;

void togglePower() {
  if ((millis() - powerFlipRequestTime) > 200)
  {
    volumeControl.setGain(0);
    digitalWrite(MutePin, HIGH);
    delay(100);
    
    if (powerState == 0) {
      powerState = 1;
      digitalWrite(PowerPin, HIGH);        
      
    } else {
        powerState = 0;
        digitalWrite(PowerPin, LOW);        
    }
    delay(1000);
    if (powerState == 1) {
        digitalWrite(MutePin, LOW);
        volumeControl.setGain(configuration.gain);
    }
  }
  powerFlipRequestTime = millis();
}

void setup() {

   lcd.begin(16, 2);

   irReceiver.enableIRIn();
   SPI.begin();

   EEPROM_readAnything(0, configuration);

   changeInput(configuration.activeInput);
   printInput(configuration.activeInput);
   volumeControl.setGain(configuration.gain);

   printGain(configuration.gain);

   pinMode(PowerPin, OUTPUT);
   digitalWrite(PowerPin, LOW);

   pinMode(MutePin, OUTPUT);
   digitalWrite(MutePin, HIGH);
}

void loop() {


    if (irReceiver.decode(&results)) {

      switch (results.value) {
        case VOLUME_UP_CODE:
          changeGain(GainUp);
          break;
        case VOLUME_DOWN_CODE:
          changeGain(GainDown);
          break;
        case INPUT_1:
          changeInput(InputAnalog);
          break;
        case INPUT_2:
          changeInput(InputOptical);
          break;
        case INPUT_3:
          changeInput(InputCoaxial);
          break;
        case INPUT_4:
          changeInput(InputAirplay);
          break;
        case INPUT_5:
          changeInput(InputBluetooth);
          break;
        case POWER_CODE:
          togglePower();
          break;
      }
   
      irReceiver.resume();
    }

}
